@@ -15,12 +15,12 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 import json
 import logging
 import warnings
 from json import JSONDecodeError
-from typing import Dict, Optional, Union
 from urllib.parse import parse_qsl, quote, unquote, urlencode, urlparse
 
 from sqlalchemy import Boolean, Column, Integer, String, Text
@@ -28,10 +28,9 @@ from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import reconstructor, synonym
 
 from airflow.configuration import ensure_secrets_loaded
-from airflow.exceptions import AirflowException, AirflowNotFoundException
+from airflow.exceptions import AirflowException, AirflowNotFoundException, RemovedInAirflow3Warning
 from airflow.models.base import ID_LEN, Base
 from airflow.models.crypto import get_fernet
-from airflow.providers_manager import ProvidersManager
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.log.secrets_masker import mask_secret
 from airflow.utils.module_loading import import_string
@@ -41,7 +40,7 @@ log = logging.getLogger(__name__)
 
 def parse_netloc_to_hostname(*args, **kwargs):
     """This method is deprecated."""
-    warnings.warn("This method is deprecated.", DeprecationWarning)
+    warnings.warn("This method is deprecated.", RemovedInAirflow3Warning)
     return _parse_netloc_to_hostname(*args, **kwargs)
 
 
@@ -90,7 +89,7 @@ class Connection(Base, LoggingMixin):
     id = Column(Integer(), primary_key=True)
     conn_id = Column(String(ID_LEN), unique=True, nullable=False)
     conn_type = Column(String(500), nullable=False)
-    description = Column(Text(5000))
+    description = Column(Text().with_variant(Text(5000), 'mysql').with_variant(String(5000), 'sqlite'))
     host = Column(String(500))
     schema = Column(String(500))
     login = Column(String(500))
@@ -102,16 +101,16 @@ class Connection(Base, LoggingMixin):
 
     def __init__(
         self,
-        conn_id: Optional[str] = None,
-        conn_type: Optional[str] = None,
-        description: Optional[str] = None,
-        host: Optional[str] = None,
-        login: Optional[str] = None,
-        password: Optional[str] = None,
-        schema: Optional[str] = None,
-        port: Optional[int] = None,
-        extra: Optional[Union[str, dict]] = None,
-        uri: Optional[str] = None,
+        conn_id: str | None = None,
+        conn_type: str | None = None,
+        description: str | None = None,
+        host: str | None = None,
+        login: str | None = None,
+        password: str | None = None,
+        schema: str | None = None,
+        port: int | None = None,
+        extra: str | dict | None = None,
+        uri: str | None = None,
     ):
         super().__init__()
         self.conn_id = conn_id
@@ -155,14 +154,14 @@ class Connection(Base, LoggingMixin):
                     "Encountered JSON value in `extra` which does not parse as a dictionary in "
                     f"connection {conn_id!r}. From Airflow 3.0, the `extra` field must contain a JSON "
                     "representation of a Python dict.",
-                    DeprecationWarning,
+                    RemovedInAirflow3Warning,
                     stacklevel=3,
                 )
         except json.JSONDecodeError:
             warnings.warn(
                 f"Encountered non-JSON in `extra` field for connection {conn_id!r}. Support for "
                 "non-JSON `extra` will be removed in Airflow 3.0",
-                DeprecationWarning,
+                RemovedInAirflow3Warning,
                 stacklevel=2,
             )
         return None
@@ -175,7 +174,8 @@ class Connection(Base, LoggingMixin):
     def parse_from_uri(self, **uri):
         """This method is deprecated. Please use uri parameter in constructor."""
         warnings.warn(
-            "This method is deprecated. Please use uri parameter in constructor.", DeprecationWarning
+            "This method is deprecated. Please use uri parameter in constructor.",
+            RemovedInAirflow3Warning,
         )
         self._parse_from_uri(**uri)
 
@@ -208,8 +208,8 @@ class Connection(Base, LoggingMixin):
         """Return connection in URI format"""
         if '_' in self.conn_type:
             self.log.warning(
-                f"Connection schemes (type: {str(self.conn_type)}) "
-                f"shall not contain '_' according to RFC3986."
+                "Connection schemes (type: %s) shall not contain '_' according to RFC3986.",
+                self.conn_type,
             )
 
         uri = f"{str(self.conn_type).lower().replace('_', '-')}://"
@@ -231,10 +231,10 @@ class Connection(Base, LoggingMixin):
             host_block += quote(self.host, safe='')
 
         if self.port:
-            if host_block > '':
-                host_block += f':{self.port}'
-            else:
+            if host_block == '' and authority_block == '':
                 host_block += f'@:{self.port}'
+            else:
+                host_block += f':{self.port}'
 
         if self.schema:
             host_block += f"/{quote(self.schema, safe='')}"
@@ -243,17 +243,17 @@ class Connection(Base, LoggingMixin):
 
         if self.extra:
             try:
-                query: Optional[str] = urlencode(self.extra_dejson)
+                query: str | None = urlencode(self.extra_dejson)
             except TypeError:
                 query = None
             if query and self.extra_dejson == dict(parse_qsl(query, keep_blank_values=True)):
-                uri += '?' + query
+                uri += ('?' if self.schema else '/?') + query
             else:
-                uri += '?' + urlencode({self.EXTRA_KEY: self.extra})
+                uri += ('?' if self.schema else '/?') + urlencode({self.EXTRA_KEY: self.extra})
 
         return uri
 
-    def get_password(self) -> Optional[str]:
+    def get_password(self) -> str | None:
         """Return encrypted password."""
         if self._password and self.is_encrypted:
             fernet = get_fernet()
@@ -266,7 +266,7 @@ class Connection(Base, LoggingMixin):
         else:
             return self._password
 
-    def set_password(self, value: Optional[str]):
+    def set_password(self, value: str | None):
         """Encrypt password and set in object attribute."""
         if value:
             fernet = get_fernet()
@@ -278,7 +278,7 @@ class Connection(Base, LoggingMixin):
         """Password. The value is decrypted/encrypted when reading/setting the value."""
         return synonym('_password', descriptor=property(cls.get_password, cls.set_password))
 
-    def get_extra(self) -> Dict:
+    def get_extra(self) -> dict:
         """Return encrypted extra-data."""
         if self._extra and self.is_extra_encrypted:
             fernet = get_fernet()
@@ -320,6 +320,8 @@ class Connection(Base, LoggingMixin):
 
     def get_hook(self, *, hook_params=None):
         """Return hook based on conn_type"""
+        from airflow.providers_manager import ProvidersManager
+
         hook = ProvidersManager().hooks.get(self.conn_type, None)
 
         if hook is None:
@@ -349,7 +351,7 @@ class Connection(Base, LoggingMixin):
         warnings.warn(
             "This method is deprecated. You can read each field individually or "
             "use the default representation (__repr__).",
-            DeprecationWarning,
+            RemovedInAirflow3Warning,
             stacklevel=2,
         )
         return (
@@ -366,7 +368,7 @@ class Connection(Base, LoggingMixin):
         warnings.warn(
             "This method is deprecated. You can read each field individually or "
             "use the default representation (__repr__).",
-            DeprecationWarning,
+            RemovedInAirflow3Warning,
             stacklevel=2,
         )
         return (
@@ -392,7 +394,7 @@ class Connection(Base, LoggingMixin):
         return status, message
 
     @property
-    def extra_dejson(self) -> Dict:
+    def extra_dejson(self) -> dict:
         """Returns the extra property by deserializing json."""
         obj = {}
         if self.extra:
@@ -408,7 +410,7 @@ class Connection(Base, LoggingMixin):
         return obj
 
     @classmethod
-    def get_connection_from_secrets(cls, conn_id: str) -> 'Connection':
+    def get_connection_from_secrets(cls, conn_id: str) -> Connection:
         """
         Get connection by conn_id.
 
@@ -430,7 +432,7 @@ class Connection(Base, LoggingMixin):
         raise AirflowNotFoundException(f"The conn_id `{conn_id}` isn't defined")
 
     @classmethod
-    def from_json(cls, value, conn_id=None) -> 'Connection':
+    def from_json(cls, value, conn_id=None) -> Connection:
         kwargs = json.loads(value)
         extra = kwargs.pop('extra', None)
         if extra:
