@@ -14,23 +14,20 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from typing import Any, Dict, Optional
+from __future__ import annotations
 
-import jsonschema
-from jsonschema import FormatChecker
-from jsonschema.exceptions import ValidationError
+import contextlib
+import copy
+import json
+import warnings
+from typing import TYPE_CHECKING, Any, ItemsView, MutableMapping, ValuesView
 
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, ParamValidationError, RemovedInAirflow3Warning
+from airflow.utils.context import Context
+from airflow.utils.types import NOTSET, ArgNotSet
 
-
-class NoValueSentinel:
-    """Sentinel class used to distinguish between None and no passed value"""
-
-    def __str__(self):
-        return "NoValueSentinel"
-
-    def __repr__(self):
-        return "NoValueSentinel"
+if TYPE_CHECKING:
+    from airflow.models.dag import DAG
 
 
 class Param:
@@ -39,52 +36,63 @@ class Param:
     it always validates and returns the default value.
 
     :param default: The value this Param object holds
-    :type default: Any
     :param description: Optional help text for the Param
-    :type description: str
     :param schema: The validation schema of the Param, if not given then all kwargs except
         default & description will form the schema
-    :type schema: dict
     """
 
-    __NO_VALUE_SENTINEL = NoValueSentinel()
     CLASS_IDENTIFIER = '__class'
 
-    def __init__(self, default: Any = __NO_VALUE_SENTINEL, description: str = None, **kwargs):
+    def __init__(self, default: Any = NOTSET, description: str | None = None, **kwargs):
+        if default is not NOTSET:
+            self._warn_if_not_json(default)
         self.value = default
         self.description = description
         self.schema = kwargs.pop('schema') if 'schema' in kwargs else kwargs
 
-        # If we have a value, validate it once. May raise ValueError.
-        if self.has_value:
-            try:
-                jsonschema.validate(self.value, self.schema, format_checker=FormatChecker())
-            except ValidationError as err:
-                raise ValueError(err)
+    def __copy__(self) -> Param:
+        return Param(self.value, self.description, schema=self.schema)
 
-    def resolve(self, value: Optional[Any] = __NO_VALUE_SENTINEL, suppress_exception: bool = False) -> Any:
+    @staticmethod
+    def _warn_if_not_json(value):
+        try:
+            json.dumps(value)
+        except Exception:
+            warnings.warn(
+                "The use of non-json-serializable params is deprecated and will be removed in "
+                "a future release",
+                RemovedInAirflow3Warning,
+            )
+
+    def resolve(self, value: Any = NOTSET, suppress_exception: bool = False) -> Any:
         """
         Runs the validations and returns the Param's final value.
         May raise ValueError on failed validations, or TypeError
         if no value is passed and no value already exists.
+        We first check that value is json-serializable; if not, warn.
+        In future release we will require the value to be json-serializable.
 
         :param value: The value to be updated for the Param
-        :type value: Optional[Any]
         :param suppress_exception: To raise an exception or not when the validations fails.
             If true and validations fails, the return value would be None.
-        :type suppress_exception: bool
         """
-        final_val = value if value != self.__NO_VALUE_SENTINEL else self.value
-        if isinstance(final_val, NoValueSentinel):
+        import jsonschema
+        from jsonschema import FormatChecker
+        from jsonschema.exceptions import ValidationError
+
+        if value is not NOTSET:
+            self._warn_if_not_json(value)
+        final_val = value if value is not NOTSET else self.value
+        if isinstance(final_val, ArgNotSet):
             if suppress_exception:
                 return None
-            raise TypeError("No value passed and Param has no default value")
+            raise ParamValidationError("No value passed and Param has no default value")
         try:
             jsonschema.validate(final_val, self.schema, format_checker=FormatChecker())
         except ValidationError as err:
             if suppress_exception:
                 return None
-            raise ValueError(err) from None
+            raise ParamValidationError(err) from None
         self.value = final_val
         return final_val
 
@@ -96,55 +104,76 @@ class Param:
 
     @property
     def has_value(self) -> bool:
-        return not isinstance(self.value, NoValueSentinel)
+        return self.value is not NOTSET
 
 
-class ParamsDict(dict):
+class ParamsDict(MutableMapping[str, Any]):
     """
     Class to hold all params for dags or tasks. All the keys are strictly string and values
     are converted into Param's object if they are not already. This class is to replace param's
     dictionary implicitly and ideally not needed to be used directly.
     """
 
-    def __init__(self, dict_obj: Optional[Dict] = None, suppress_exception: bool = False):
+    __slots__ = ['__dict', 'suppress_exception']
+
+    def __init__(self, dict_obj: dict | None = None, suppress_exception: bool = False):
         """
-        Init override for ParamsDict
         :param dict_obj: A dict or dict like object to init ParamsDict
-        :type dict_obj: Optional[dict]
         :param suppress_exception: Flag to suppress value exceptions while initializing the ParamsDict
-        :type suppress_exception: bool
         """
-        params_dict = {}
+        params_dict: dict[str, Param] = {}
         dict_obj = dict_obj or {}
         for k, v in dict_obj.items():
             if not isinstance(v, Param):
                 params_dict[k] = Param(v)
             else:
                 params_dict[k] = v
-        super().__init__(params_dict)
+        self.__dict = params_dict
         self.suppress_exception = suppress_exception
 
+    def __copy__(self) -> ParamsDict:
+        return ParamsDict(self.__dict, self.suppress_exception)
+
+    def __deepcopy__(self, memo: dict[int, Any] | None) -> ParamsDict:
+        return ParamsDict(copy.deepcopy(self.__dict, memo), self.suppress_exception)
+
+    def __contains__(self, o: object) -> bool:
+        return o in self.__dict
+
+    def __len__(self) -> int:
+        return len(self.__dict)
+
+    def __delitem__(self, v: str) -> None:
+        del self.__dict[v]
+
+    def __iter__(self):
+        return iter(self.__dict)
+
+    def __repr__(self):
+        return repr(self.dump())
+
     def __setitem__(self, key: str, value: Any) -> None:
         """
         Override for dictionary's ``setitem`` method. This method make sure that all values are of
         Param's type only.
 
         :param key: A key which needs to be inserted or updated in the dict
-        :type key: str
         :param value: A value which needs to be set against the key. It could be of any
             type but will be converted and stored as a Param object eventually.
-        :type value: Any
         """
         if isinstance(value, Param):
             param = value
-        elif key in self:
-            param = dict.__getitem__(self, key)
-            param.resolve(value=value, suppress_exception=self.suppress_exception)
+        elif key in self.__dict:
+            param = self.__dict[key]
+            try:
+                param.resolve(value=value, suppress_exception=self.suppress_exception)
+            except ParamValidationError as ve:
+                raise ParamValidationError(f'Invalid input for param {key}: {ve}') from None
         else:
             # if the key isn't there already and if the value isn't of Param type create a new Param object
             param = Param(value)
 
-        super().__setitem__(key, param)
+        self.__dict[key] = param
 
     def __getitem__(self, key: str) -> Any:
         """
@@ -152,35 +181,37 @@ class ParamsDict(dict):
         resolve method as well on the Param object.
 
         :param key: The key to fetch
-        :type key: str
         """
-        param = super().__getitem__(key)
+        param = self.__dict[key]
         return param.resolve(suppress_exception=self.suppress_exception)
 
-    def dump(self) -> dict:
+    def get_param(self, key: str) -> Param:
+        """Get the internal :class:`.Param` object for this key"""
+        return self.__dict[key]
+
+    def items(self):
+        return ItemsView(self.__dict)
+
+    def values(self):
+        return ValuesView(self.__dict)
+
+    def update(self, *args, **kwargs) -> None:
+        if len(args) == 1 and not kwargs and isinstance(args[0], ParamsDict):
+            return super().update(args[0].__dict)
+        super().update(*args, **kwargs)
+
+    def dump(self) -> dict[str, Any]:
         """Dumps the ParamsDict object as a dictionary, while suppressing exceptions"""
         return {k: v.resolve(suppress_exception=True) for k, v in self.items()}
 
-    def update(self, other_dict: dict) -> None:
-        """
-        Override for dictionary's update method.
-        :param other_dict: A dict type object which needs to be merged in the ParamsDict object
-        :type other_dict: dict
-        """
-        try:
-            for k, v in other_dict.items():
-                self.__setitem__(k, v)
-        except ValueError as ve:
-            raise ValueError(f'Invalid input for param {k}: {ve}') from None
-
-    def validate(self) -> dict:
+    def validate(self) -> dict[str, Any]:
         """Validates & returns all the Params object stored in the dictionary"""
         resolved_dict = {}
         try:
-            for k, v in dict.items(self):
+            for k, v in self.items():
                 resolved_dict[k] = v.resolve(suppress_exception=self.suppress_exception)
-        except ValueError as ve:
-            raise ValueError(f'Invalid input for param {k}: {ve}') from None
+        except ParamValidationError as ve:
+            raise ParamValidationError(f'Invalid input for param {k}: {ve}') from None
 
         return resolved_dict
 
@@ -202,25 +233,22 @@ class DagParam:
           EmailOperator(subject=dag.param('subject', 'Hi from Airflow!'))
 
     :param current_dag: Dag being used for parameter.
-    :type current_dag: airflow.models.DAG
     :param name: key value which is used to set the parameter
-    :type name: str
     :param default: Default value used if no parameter was set.
-    :type default: Any
     """
 
-    def __init__(self, current_dag, name: str, default: Optional[Any] = None):
-        if default:
+    def __init__(self, current_dag: DAG, name: str, default: Any = NOTSET):
+        if default is not NOTSET:
             current_dag.params[name] = default
         self._name = name
         self._default = default
 
-    def resolve(self, context: Dict) -> Any:
+    def resolve(self, context: Context) -> Any:
         """Pull DagParam value from DagRun context. This method is run during ``op.execute()``."""
-        default = self._default
-        if not self._default:
-            default = context['params'][self._name] if self._name in context['params'] else None
-        resolved = context['dag_run'].conf.get(self._name, default)
-        if not resolved:
-            raise AirflowException(f'No value could be resolved for parameter {self._name}')
-        return resolved
+        with contextlib.suppress(KeyError):
+            return context['dag_run'].conf[self._name]
+        if self._default is not NOTSET:
+            return self._default
+        with contextlib.suppress(KeyError):
+            return context['params'][self._name]
+        raise AirflowException(f'No value could be resolved for parameter {self._name}')
