@@ -23,7 +23,7 @@ operators talk to the ``api/2.0/jobs/runs/submit``
 `endpoint <https://docs.databricks.com/api/latest/jobs.html#runs-submit>`_.
 """
 import copy
-import sys
+import platform
 import time
 from typing import Any, Dict, Optional, Tuple
 from urllib.parse import urlparse
@@ -43,16 +43,11 @@ from tenacity import (
 )
 
 from airflow import __version__
+from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException
 from airflow.hooks.base import BaseHook
 from airflow.models import Connection
-
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-    from cached_property import cached_property
-
-USER_AGENT_HEADER = {'user-agent': f'airflow-{__version__}'}
+from airflow.providers_manager import ProvidersManager
 
 # https://docs.microsoft.com/en-us/azure/databricks/dev-tools/api/latest/aad/service-prin-aad-token#--get-an-azure-active-directory-access-token
 # https://docs.microsoft.com/en-us/graph/deployments#app-registration-and-token-service-root-endpoints
@@ -101,6 +96,7 @@ class BaseDatabricksHook(BaseHook):
         retry_limit: int = 3,
         retry_delay: float = 1.0,
         retry_args: Optional[Dict[Any, Any]] = None,
+        caller: str = "Unknown",
     ) -> None:
         super().__init__()
         self.databricks_conn_id = databricks_conn_id
@@ -111,6 +107,7 @@ class BaseDatabricksHook(BaseHook):
         self.retry_delay = retry_delay
         self.aad_tokens: Dict[str, dict] = {}
         self.aad_timeout_seconds = 10
+        self.caller = caller
 
         def my_after_func(retry_state):
             self._log_request_error(retry_state.attempt_number, retry_state.outcome)
@@ -134,6 +131,24 @@ class BaseDatabricksHook(BaseHook):
     def get_conn(self) -> Connection:
         return self.databricks_conn
 
+    @cached_property
+    def user_agent_header(self) -> Dict[str, str]:
+        return {'user-agent': self.user_agent_value}
+
+    @cached_property
+    def user_agent_value(self) -> str:
+        manager = ProvidersManager()
+        package_name = manager.hooks[BaseDatabricksHook.conn_type].package_name  # type: ignore[union-attr]
+        provider = manager.providers[package_name]
+        version = provider.version
+        python_version = platform.python_version()
+        system = platform.system().lower()
+        ua_string = (
+            f"databricks-airflow/{version} _/0.0.0 python/{python_version} os/{system} "
+            f"airflow/{__version__} operator/{self.caller}"
+        )
+        return ua_string
+
     @cached_property
     def host(self) -> str:
         if 'host' in self.databricks_conn.extra_dejson:
@@ -154,8 +169,7 @@ class BaseDatabricksHook(BaseHook):
     @staticmethod
     def _parse_host(host: str) -> str:
         """
-        The purpose of this function is to be robust to improper connections
-        settings provided by users, specifically in the host field.
+        This function is resistant to incorrect connection settings provided by users, in the host field.
 
         For example -- when users supply ``https://xx.cloud.databricks.com`` as the
         host, we must strip out the protocol to get the host.::
@@ -180,21 +194,23 @@ class BaseDatabricksHook(BaseHook):
 
     def _get_retry_object(self) -> Retrying:
         """
-        Instantiates a retry object
+        Instantiate a retry object.
         :return: instance of Retrying class
         """
         return Retrying(**self.retry_args)
 
     def _a_get_retry_object(self) -> AsyncRetrying:
         """
-        Instantiates an async retry object
+        Instantiate an async retry object.
         :return: instance of AsyncRetrying class
         """
         return AsyncRetrying(**self.retry_args)
 
     def _get_aad_token(self, resource: str) -> str:
         """
-        Function to get AAD token for given resource. Supports managed identity or service principal auth
+        Function to get AAD token for given resource.
+
+        Supports managed identity or service principal auth.
         :param resource: resource to issue token to
         :return: AAD token, or raise an exception
         """
@@ -214,7 +230,7 @@ class BaseDatabricksHook(BaseHook):
                         resp = requests.get(
                             AZURE_METADATA_SERVICE_TOKEN_URL,
                             params=params,
-                            headers={**USER_AGENT_HEADER, "Metadata": "true"},
+                            headers={**self.user_agent_header, "Metadata": "true"},
                             timeout=self.aad_timeout_seconds,
                         )
                     else:
@@ -232,7 +248,7 @@ class BaseDatabricksHook(BaseHook):
                             AZURE_TOKEN_SERVICE_URL.format(azure_ad_endpoint, tenant_id),
                             data=data,
                             headers={
-                                **USER_AGENT_HEADER,
+                                **self.user_agent_header,
                                 'Content-Type': 'application/x-www-form-urlencoded',
                             },
                             timeout=self.aad_timeout_seconds,
@@ -279,7 +295,7 @@ class BaseDatabricksHook(BaseHook):
                         async with self._session.get(
                             url=AZURE_METADATA_SERVICE_TOKEN_URL,
                             params=params,
-                            headers={**USER_AGENT_HEADER, "Metadata": "true"},
+                            headers={**self.user_agent_header, "Metadata": "true"},
                             timeout=self.aad_timeout_seconds,
                         ) as resp:
                             resp.raise_for_status()
@@ -299,7 +315,7 @@ class BaseDatabricksHook(BaseHook):
                             url=AZURE_TOKEN_SERVICE_URL.format(azure_ad_endpoint, tenant_id),
                             data=data,
                             headers={
-                                **USER_AGENT_HEADER,
+                                **self.user_agent_header,
                                 'Content-Type': 'application/x-www-form-urlencoded',
                             },
                             timeout=self.aad_timeout_seconds,
@@ -325,7 +341,7 @@ class BaseDatabricksHook(BaseHook):
 
     def _get_aad_headers(self) -> dict:
         """
-        Fills AAD headers if necessary (SPN is outside of the workspace)
+        Fill AAD headers if necessary (SPN is outside of the workspace).
         :return: dictionary with filled AAD headers
         """
         headers = {}
@@ -354,7 +370,8 @@ class BaseDatabricksHook(BaseHook):
     @staticmethod
     def _is_aad_token_valid(aad_token: dict) -> bool:
         """
-        Utility function to check AAD token hasn't expired yet
+        Utility function to check AAD token hasn't expired yet.
+
         :param aad_token: dict with properties of AAD token
         :return: true if token is valid, false otherwise
         :rtype: bool
@@ -367,7 +384,7 @@ class BaseDatabricksHook(BaseHook):
     @staticmethod
     def _check_azure_metadata_service() -> None:
         """
-        Check for Azure Metadata Service
+        Check for Azure Metadata Service.
         https://docs.microsoft.com/en-us/azure/virtual-machines/linux/instance-metadata-service
         """
         try:
@@ -457,7 +474,7 @@ class BaseDatabricksHook(BaseHook):
         wrap_http_errors: bool = True,
     ):
         """
-        Utility function to perform an API call with retries
+        Utility function to perform an API call with retries.
 
         :param endpoint_info: Tuple of method and endpoint
         :param json: Parameters for this API call.
@@ -472,7 +489,7 @@ class BaseDatabricksHook(BaseHook):
         url = f'https://{self.host}/{endpoint}'
 
         aad_headers = self._get_aad_headers()
-        headers = {**USER_AGENT_HEADER.copy(), **aad_headers}
+        headers = {**self.user_agent_header, **aad_headers}
 
         auth: AuthBase
         token = self._get_token()
@@ -530,7 +547,7 @@ class BaseDatabricksHook(BaseHook):
         url = f'https://{self.host}/{endpoint}'
 
         aad_headers = await self._a_get_aad_headers()
-        headers = {**USER_AGENT_HEADER.copy(), **aad_headers}
+        headers = {**self.user_agent_header, **aad_headers}
 
         auth: aiohttp.BasicAuth
         token = await self._a_get_token()
@@ -556,7 +573,7 @@ class BaseDatabricksHook(BaseHook):
                         url,
                         json=json,
                         auth=auth,
-                        headers={**headers, **USER_AGENT_HEADER},
+                        headers={**headers, **self.user_agent_header},
                         timeout=self.timeout_seconds,
                     ) as response:
                         response.raise_for_status()
@@ -602,7 +619,9 @@ class BaseDatabricksHook(BaseHook):
 
 class _TokenAuth(AuthBase):
     """
-    Helper class for requests Auth field. AuthBase requires you to implement the __call__
+    Helper class for requests Auth field.
+
+    AuthBase requires you to implement the ``__call__``
     magic function.
     """
 
