@@ -42,7 +42,7 @@ class DataprocMetastoreHook(GoogleBaseHook):
         client_options = ClientOptions(api_endpoint='metastore.googleapis.com:443')
 
         return DataprocMetastoreClient(
-            credentials=self._get_credentials(), client_info=CLIENT_INFO, client_options=client_options
+            credentials=self.get_credentials(), client_info=CLIENT_INFO, client_options=client_options
         )
 
     def wait_for_operation(self, timeout: Optional[float], operation: Operation):
