@@ -16,46 +16,43 @@
 # specific language governing permissions and limitations
 # under the License.
 """Manages all providers."""
+from __future__ import annotations
+
 import fnmatch
 import functools
-import importlib
 import json
 import logging
 import os
 import sys
 import warnings
 from collections import OrderedDict
+from dataclasses import dataclass
 from functools import wraps
 from time import perf_counter
-from typing import Any, Callable, Dict, List, MutableMapping, NamedTuple, Optional, Set, TypeVar, Union, cast
-
-import jsonschema
-from packaging import version as packaging_version
+from typing import TYPE_CHECKING, Any, Callable, MutableMapping, NamedTuple, TypeVar, cast
 
+from airflow.exceptions import AirflowOptionalProviderFeatureException
+from airflow.typing_compat import Literal
 from airflow.utils import yaml
 from airflow.utils.entry_points import entry_points_with_dist
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.module_loading import import_string
 
-try:
-    import importlib.resources as importlib_resources
-except ImportError:
-    # Try back-ported to PY<37 `importlib_resources`.
-    import importlib_resources
-
 log = logging.getLogger(__name__)
 
 if sys.version_info >= (3, 9):
-    from functools import cache
+    from importlib.resources import files as resource_files
 else:
-    from functools import lru_cache
-
-    cache = lru_cache(maxsize=None)
+    from importlib_resources import files as resource_files
 
 MIN_PROVIDER_VERSIONS = {
     "apache-airflow-providers-celery": "2.1.0",
 }
 
+if TYPE_CHECKING:
+    from airflow.decorators.base import TaskDecorator
+    from airflow.hooks.base import BaseHook
+
 
 class LazyDictWithCache(MutableMapping):
     """
@@ -102,7 +99,10 @@ class LazyDictWithCache(MutableMapping):
 
 def _create_provider_info_schema_validator():
     """Creates JSON schema validator from the provider_info.schema.json"""
-    schema = json.loads(importlib_resources.read_text('airflow', 'provider_info.schema.json'))
+    import jsonschema
+
+    with resource_files("airflow").joinpath("provider_info.schema.json").open("rb") as f:
+        schema = json.load(f)
     cls = jsonschema.validators.validator_for(schema)
     validator = cls(schema)
     return validator
@@ -110,9 +110,10 @@ def _create_provider_info_schema_validator():
 
 def _create_customized_form_field_behaviours_schema_validator():
     """Creates JSON schema validator from the customized_form_field_behaviours.schema.json"""
-    schema = json.loads(
-        importlib_resources.read_text('airflow', 'customized_form_field_behaviours.schema.json')
-    )
+    import jsonschema
+
+    with resource_files("airflow").joinpath("customized_form_field_behaviours.schema.json").open("rb") as f:
+        schema = json.load(f)
     cls = jsonschema.validators.validator_for(schema)
     validator = cls(schema)
     return validator
@@ -132,38 +133,28 @@ def _check_builtin_provider_prefix(provider_package: str, class_name: str) -> bo
     return True
 
 
-def _sanity_check(provider_package: str, class_name: str) -> bool:
+@dataclass
+class ProviderInfo:
     """
-    Performs coherence check on provider classes.
-    For apache-airflow providers - it checks if it starts with appropriate package. For all providers
-    it tries to import the provider - checking that there are no exceptions during importing.
-    It logs appropriate warning in case it detects any problems.
+    Provider information
 
-    :param provider_package: name of the provider package
-    :param class_name: name of the class to import
-
-    :return True if the class is OK, False otherwise.
+    :param version: version string
+    :param data: dictionary with information about the provider
+    :param source_or_package: whether the provider is source files or PyPI package. When installed from
+        sources we suppress provider import errors.
     """
-    if not _check_builtin_provider_prefix(provider_package, class_name):
-        return False
-    try:
-        import_string(class_name)
-    except Exception as e:
-        log.warning(
-            "Exception when importing '%s' from '%s' package: %s",
-            class_name,
-            provider_package,
-            e,
-        )
-        return False
-    return True
-
-
-class ProviderInfo(NamedTuple):
-    """Provider information"""
 
     version: str
-    provider_info: Dict
+    data: dict
+    package_or_source: Literal['source'] | Literal['package']
+
+    def __post_init__(self):
+        if self.package_or_source not in ('source', 'package'):
+            raise ValueError(
+                f"Received {self.package_or_source!r} for `package_or_source`. "
+                "Must be either 'package' or 'source'."
+            )
+        self.is_source = self.package_or_source == 'source'
 
 
 class HookClassProvider(NamedTuple):
@@ -181,6 +172,7 @@ class HookInfo(NamedTuple):
     package_name: str
     hook_name: str
     connection_type: str
+    connection_testable: bool
 
 
 class ConnectionFormWidgetInfo(NamedTuple):
@@ -189,6 +181,7 @@ class ConnectionFormWidgetInfo(NamedTuple):
     hook_class_name: str
     package_name: str
     field: Any
+    field_name: str
 
 
 T = TypeVar("T", bound=Callable)
@@ -196,6 +189,98 @@ T = TypeVar("T", bound=Callable)
 logger = logging.getLogger(__name__)
 
 
+def log_debug_import_from_sources(class_name, e, provider_package):
+    log.debug(
+        "Optional feature disabled on exception when importing '%s' from '%s' package",
+        class_name,
+        provider_package,
+        exc_info=e,
+    )
+
+
+def log_optional_feature_disabled(class_name, e, provider_package):
+    log.debug(
+        "Optional feature disabled on exception when importing '%s' from '%s' package",
+        class_name,
+        provider_package,
+        exc_info=e,
+    )
+    log.info(
+        "Optional provider feature disabled when importing '%s' from '%s' package",
+        class_name,
+        provider_package,
+    )
+
+
+def log_import_warning(class_name, e, provider_package):
+    log.warning(
+        "Exception when importing '%s' from '%s' package",
+        class_name,
+        provider_package,
+        exc_info=e,
+    )
+
+
+# This is a temporary measure until all community providers will add AirflowOptionalProviderFeatureException
+# where they have optional features. We are going to add tests in our CI to catch all such cases and will
+# fix them, but until now all "known unhandled optional feature errors" from community providers
+# should be added here
+KNOWN_UNHANDLED_OPTIONAL_FEATURE_ERRORS = [("apache-airflow-providers-google", "No module named 'paramiko'")]
+
+
+def _sanity_check(
+    provider_package: str, class_name: str, provider_info: ProviderInfo
+) -> type[BaseHook] | None:
+    """
+    Performs coherence check on provider classes.
+    For apache-airflow providers - it checks if it starts with appropriate package. For all providers
+    it tries to import the provider - checking that there are no exceptions during importing.
+    It logs appropriate warning in case it detects any problems.
+
+    :param provider_package: name of the provider package
+    :param class_name: name of the class to import
+
+    :return the class if the class is OK, None otherwise.
+    """
+    if not _check_builtin_provider_prefix(provider_package, class_name):
+        return None
+    try:
+        imported_class = import_string(class_name)
+    except AirflowOptionalProviderFeatureException as e:
+        # When the provider class raises AirflowOptionalProviderFeatureException
+        # this is an expected case when only some classes in provider are
+        # available. We just log debug level here and print info message in logs so that
+        # the user is aware of it
+        log_optional_feature_disabled(class_name, e, provider_package)
+        return None
+    except ImportError as e:
+        if provider_info.is_source:
+            # When we have providers from sources, then we just turn all import logs to debug logs
+            # As this is pretty expected that you have a number of dependencies not installed
+            # (we always have all providers from sources until we split providers to separate repo)
+            log_debug_import_from_sources(class_name, e, provider_package)
+            return None
+        if "No module named 'airflow.providers." in e.msg:
+            # handle cases where another provider is missing. This can only happen if
+            # there is an optional feature, so we log debug and print information about it
+            log_optional_feature_disabled(class_name, e, provider_package)
+            return None
+        for known_error in KNOWN_UNHANDLED_OPTIONAL_FEATURE_ERRORS:
+            # Until we convert all providers to use AirflowOptionalProviderFeatureException
+            # we assume any problem with importing another "provider" is because this is an
+            # optional feature, so we log debug and print information about it
+            if known_error[0] == provider_package and known_error[1] in e.msg:
+                log_optional_feature_disabled(class_name, e, provider_package)
+                return None
+        # But when we have no idea - we print warning to logs
+        log_import_warning(class_name, e, provider_package)
+        return None
+    except Exception as e:
+        log_import_warning(class_name, e, provider_package)
+        return None
+    return imported_class
+
+
 # We want to have better control over initialization of parameters and be able to debug and test it
 # So we add our own decorator
 def provider_info_cache(cache_name: str) -> Callable[[T], T]:
@@ -243,25 +328,25 @@ class ProvidersManager(LoggingMixin):
     def __init__(self):
         """Initializes the manager."""
         super().__init__()
-        self._initialized_cache: Dict[str, bool] = {}
+        self._initialized_cache: dict[str, bool] = {}
         # Keeps dict of providers keyed by module name
-        self._provider_dict: Dict[str, ProviderInfo] = {}
+        self._provider_dict: dict[str, ProviderInfo] = {}
         # Keeps dict of hooks keyed by connection type
-        self._hooks_dict: Dict[str, HookInfo] = {}
+        self._hooks_dict: dict[str, HookInfo] = {}
 
-        self._taskflow_decorators: Dict[str, Callable] = LazyDictWithCache()
+        self._taskflow_decorators: dict[str, Callable] = LazyDictWithCache()
         # keeps mapping between connection_types and hook class, package they come from
-        self._hook_provider_dict: Dict[str, HookClassProvider] = {}
+        self._hook_provider_dict: dict[str, HookClassProvider] = {}
         # Keeps dict of hooks keyed by connection type. They are lazy evaluated at access time
-        self._hooks_lazy_dict: LazyDictWithCache[str, Union[HookInfo, Callable]] = LazyDictWithCache()
+        self._hooks_lazy_dict: LazyDictWithCache[str, HookInfo | Callable] = LazyDictWithCache()
         # Keeps methods that should be used to add custom widgets tuple of keyed by name of the extra field
-        self._connection_form_widgets: Dict[str, ConnectionFormWidgetInfo] = {}
+        self._connection_form_widgets: dict[str, ConnectionFormWidgetInfo] = {}
         # Customizations for javascript fields are kept here
-        self._field_behaviours: Dict[str, Dict] = {}
-        self._extra_link_class_name_set: Set[str] = set()
-        self._logging_class_name_set: Set[str] = set()
-        self._secrets_backend_class_name_set: Set[str] = set()
-        self._api_auth_backend_module_names: Set[str] = set()
+        self._field_behaviours: dict[str, dict] = {}
+        self._extra_link_class_name_set: set[str] = set()
+        self._logging_class_name_set: set[str] = set()
+        self._secrets_backend_class_name_set: set[str] = set()
+        self._api_auth_backend_module_names: set[str] = set()
         self._provider_schema_validator = _create_provider_info_schema_validator()
         self._customized_form_fields_schema_validator = (
             _create_customized_form_field_behaviours_schema_validator()
@@ -280,14 +365,19 @@ class ProvidersManager(LoggingMixin):
         self._provider_dict = OrderedDict(sorted(self._provider_dict.items()))
 
     def _verify_all_providers_all_compatible(self):
+        from packaging import version as packaging_version
+
         for provider_id, info in self._provider_dict.items():
             min_version = MIN_PROVIDER_VERSIONS.get(provider_id)
             if min_version:
                 if packaging_version.parse(min_version) > packaging_version.parse(info.version):
                     log.warning(
-                        f"The package {provider_id} is not compatible with this version of Airflow. "
-                        f"The package has version {info.version} but the minimum supported version "
-                        f"of the package is {min_version}"
+                        "The package %s is not compatible with this version of Airflow. "
+                        "The package has version %s but the minimum supported version "
+                        "of the package is %s",
+                        provider_id,
+                        info.version,
+                        min_version,
                     )
 
     @provider_info_cache("hooks")
@@ -351,7 +441,7 @@ class ProvidersManager(LoggingMixin):
                     f"{provider_info_package_name} do not match. Please make sure they are aligned"
                 )
             if package_name not in self._provider_dict:
-                self._provider_dict[package_name] = ProviderInfo(version, provider_info)
+                self._provider_dict[package_name] = ProviderInfo(version, provider_info, 'package')
             else:
                 log.warning(
                     "The provider for package '%s' could not be registered from because providers for that "
@@ -374,7 +464,7 @@ class ProvidersManager(LoggingMixin):
             log.info("You have no providers installed.")
             return
         try:
-            for path in airflow.providers.__path__:
+            for path in airflow.providers.__path__:  # type: ignore[attr-defined]
                 self._add_provider_info_from_local_source_files_on_path(path)
         except Exception as e:
             log.warning("Error when loading 'provider.yaml' files from airflow sources: %s", e)
@@ -407,7 +497,7 @@ class ProvidersManager(LoggingMixin):
 
             version = provider_info['versions'][0]
             if package_name not in self._provider_dict:
-                self._provider_dict[package_name] = ProviderInfo(version, provider_info)
+                self._provider_dict[package_name] = ProviderInfo(version, provider_info, 'source')
             else:
                 log.warning(
                     "The providers for package '%s' could not be registered because providers for that "
@@ -419,8 +509,8 @@ class ProvidersManager(LoggingMixin):
 
     def _discover_hooks_from_connection_types(
         self,
-        hook_class_names_registered: Set[str],
-        already_registered_warning_connection_types: Set[str],
+        hook_class_names_registered: set[str],
+        already_registered_warning_connection_types: set[str],
         package_name: str,
         provider: ProviderInfo,
     ):
@@ -437,7 +527,7 @@ class ProvidersManager(LoggingMixin):
         :return:
         """
         provider_uses_connection_types = False
-        connection_types = provider.provider_info.get("connection-types")
+        connection_types = provider.data.get("connection-types")
         if connection_types:
             for connection_type_dict in connection_types:
                 connection_type = connection_type_dict['connection-type']
@@ -453,15 +543,17 @@ class ProvidersManager(LoggingMixin):
                     )
                     # Defer importing hook to access time by setting import hook method as dict value
                     self._hooks_lazy_dict[connection_type] = functools.partial(
-                        self._import_hook, connection_type
+                        self._import_hook,
+                        connection_type=connection_type,
+                        provider_info=provider,
                     )
             provider_uses_connection_types = True
         return provider_uses_connection_types
 
     def _discover_hooks_from_hook_class_names(
         self,
-        hook_class_names_registered: Set[str],
-        already_registered_warning_connection_types: Set[str],
+        hook_class_names_registered: set[str],
+        already_registered_warning_connection_types: set[str],
         package_name: str,
         provider: ProviderInfo,
         provider_uses_connection_types: bool,
@@ -479,7 +571,7 @@ class ProvidersManager(LoggingMixin):
            form of passing connection types
         :return:
         """
-        hook_class_names = provider.provider_info.get("hook-class-names")
+        hook_class_names = provider.data.get("hook-class-names")
         if hook_class_names:
             for hook_class_name in hook_class_names:
                 if hook_class_name in hook_class_names_registered:
@@ -487,7 +579,10 @@ class ProvidersManager(LoggingMixin):
                     # connection-types discovery
                     continue
                 hook_info = self._import_hook(
-                    connection_type=None, hook_class_name=hook_class_name, package_name=package_name
+                    connection_type=None,
+                    provider_info=provider,
+                    hook_class_name=hook_class_name,
+                    package_name=package_name,
                 )
                 if not hook_info:
                     # Problem why importing class - we ignore it. Log is written at import time
@@ -532,8 +627,8 @@ class ProvidersManager(LoggingMixin):
     def _discover_hooks(self) -> None:
         """Retrieves all connections defined in the providers via Hooks"""
         for package_name, provider in self._provider_dict.items():
-            duplicated_connection_types: Set[str] = set()
-            hook_class_names_registered: Set[str] = set()
+            duplicated_connection_types: set[str] = set()
+            hook_class_names_registered: set[str] = set()
             provider_uses_connection_types = self._discover_hooks_from_connection_types(
                 hook_class_names_registered, duplicated_connection_types, package_name, provider
             )
@@ -551,12 +646,18 @@ class ProvidersManager(LoggingMixin):
         """Force-import all hooks and initialize the connections/fields"""
         # Retrieve all hooks to make sure that all of them are imported
         _ = list(self._hooks_lazy_dict.values())
-        self._connection_form_widgets = OrderedDict(sorted(self._connection_form_widgets.items()))
         self._field_behaviours = OrderedDict(sorted(self._field_behaviours.items()))
 
+        # Widgets for connection forms are currently used in two places:
+        # 1. In the UI Connections, expected same order that it defined in Hook.
+        # 2. cli command - `airflow providers widgets` and expected that it in alphabetical order.
+        # It is not possible to recover original ordering after sorting,
+        # that the main reason why original sorting moved to cli part:
+        # self._connection_form_widgets = OrderedDict(sorted(self._connection_form_widgets.items()))
+
     def _discover_taskflow_decorators(self) -> None:
         for name, info in self._provider_dict.items():
-            for taskflow_decorator in info.provider_info.get("task-decorators", []):
+            for taskflow_decorator in info.data.get("task-decorators", []):
                 self._add_taskflow_decorator(
                     taskflow_decorator["name"], taskflow_decorator["class-name"], name
                 )
@@ -568,10 +669,10 @@ class ProvidersManager(LoggingMixin):
         if name in self._taskflow_decorators:
             try:
                 existing = self._taskflow_decorators[name]
-                other_name = f'{existing.__module__}.{existing.__name}'
+                other_name = f'{existing.__module__}.{existing.__name__}'
             except Exception:
                 # If problem importing, then get the value from the functools.partial
-                other_name = self._taskflow_decorators._raw_dict[name].args[0]
+                other_name = self._taskflow_decorators._raw_dict[name].args[0]  # type: ignore[attr-defined]
 
             log.warning(
                 "The taskflow decorator '%s' has been already registered (by %s).",
@@ -591,8 +692,12 @@ class ProvidersManager(LoggingMixin):
         return getattr(obj, attr_name)
 
     def _import_hook(
-        self, connection_type: Optional[str], hook_class_name: str = None, package_name: str = None
-    ) -> Optional[HookInfo]:
+        self,
+        connection_type: str | None,
+        provider_info: ProviderInfo,
+        hook_class_name: str | None = None,
+        package_name: str | None = None,
+    ) -> HookInfo | None:
         """
         Imports hook and retrieves hook information. Either connection_type (for lazy loading)
         or hook_class_name must be set - but not both). Only needs package_name if hook_class_name is
@@ -608,26 +713,28 @@ class ProvidersManager(LoggingMixin):
 
         if connection_type is None and hook_class_name is None:
             raise ValueError("Either connection_type or hook_class_name must be set")
-        if connection_type and hook_class_name:
+        if connection_type is not None and hook_class_name is not None:
             raise ValueError(
                 f"Both connection_type ({connection_type} and "
                 f"hook_class_name {hook_class_name} are set. Only one should be set!"
             )
-        if connection_type:
+        if connection_type is not None:
             class_provider = self._hook_provider_dict[connection_type]
             package_name = class_provider.package_name
             hook_class_name = class_provider.hook_class_name
         else:
+            if not hook_class_name:
+                raise ValueError("Either connection_type or hook_class_name must be set")
             if not package_name:
                 raise ValueError(
-                    f"Provider package name is not set when hook_class_name ({hook_class_name}) " f"is used"
+                    f"Provider package name is not set when hook_class_name ({hook_class_name}) is used"
                 )
         allowed_field_classes = [IntegerField, PasswordField, StringField, BooleanField]
-        if not _sanity_check(package_name, hook_class_name):
+        hook_class = _sanity_check(package_name, hook_class_name, provider_info)
+        if hook_class is None:
             return None
         try:
             module, class_name = hook_class_name.rsplit('.', maxsplit=1)
-            hook_class = getattr(importlib.import_module(module), class_name)
             # Do not use attr here. We want to check only direct class fields not those
             # inherited from parent hook. This way we add form fields only once for the whole
             # hierarchy and we add it only from the parent hook that provides those!
@@ -649,16 +756,6 @@ class ProvidersManager(LoggingMixin):
                 field_behaviours = hook_class.get_ui_field_behaviour()
                 if field_behaviours:
                     self._add_customized_fields(package_name, hook_class, field_behaviours)
-        except ImportError as e:
-            # When there is an ImportError we turn it into debug warnings as this is
-            # an expected case when only some providers are installed
-            log.debug(
-                "Exception when importing '%s' from '%s' package: %s",
-                hook_class_name,
-                package_name,
-                e,
-            )
-            return None
         except Exception as e:
             log.warning(
                 "Exception when importing '%s' from '%s' package: %s",
@@ -699,30 +796,29 @@ class ProvidersManager(LoggingMixin):
             package_name=package_name,
             hook_name=hook_name,
             connection_type=connection_type,
+            connection_testable=hasattr(hook_class, 'test_connection'),
         )
 
-    def _add_widgets(self, package_name: str, hook_class: type, widgets: Dict[str, Any]):
-        for field_name, field in widgets.items():
-            if not field_name.startswith("extra__"):
-                log.warning(
-                    "The field %s from class %s does not start with 'extra__'. Ignoring it.",
-                    field_name,
-                    hook_class.__name__,
-                )
-                continue
-            if field_name in self._connection_form_widgets:
+    def _add_widgets(self, package_name: str, hook_class: type, widgets: dict[str, Any]):
+        conn_type = hook_class.conn_type  # type: ignore
+        for field_identifier, field in widgets.items():
+            if field_identifier.startswith('extra__'):
+                prefixed_field_name = field_identifier
+            else:
+                prefixed_field_name = f"extra__{conn_type}__{field_identifier}"
+            if prefixed_field_name in self._connection_form_widgets:
                 log.warning(
                     "The field %s from class %s has already been added by another provider. Ignoring it.",
-                    field_name,
+                    field_identifier,
                     hook_class.__name__,
                 )
                 # In case of inherited hooks this might be happening several times
                 continue
-            self._connection_form_widgets[field_name] = ConnectionFormWidgetInfo(
-                hook_class.__name__, package_name, field
+            self._connection_form_widgets[prefixed_field_name] = ConnectionFormWidgetInfo(
+                hook_class.__name__, package_name, field, field_identifier
             )
 
-    def _add_customized_fields(self, package_name: str, hook_class: type, customized_fields: Dict):
+    def _add_customized_fields(self, package_name: str, hook_class: type, customized_fields: dict):
         try:
             connection_type = getattr(hook_class, "conn_type")
             self._customized_form_fields_schema_validator.validate(customized_fields)
@@ -746,91 +842,94 @@ class ProvidersManager(LoggingMixin):
 
     def _discover_extra_links(self) -> None:
         """Retrieves all extra links defined in the providers"""
-        for provider_package, (_, provider) in self._provider_dict.items():
-            if provider.get("extra-links"):
-                for extra_link_class_name in provider["extra-links"]:
-                    if _sanity_check(provider_package, extra_link_class_name):
+        for provider_package, provider in self._provider_dict.items():
+            if provider.data.get("extra-links"):
+                for extra_link_class_name in provider.data["extra-links"]:
+                    if _sanity_check(provider_package, extra_link_class_name, provider):
                         self._extra_link_class_name_set.add(extra_link_class_name)
 
     def _discover_logging(self) -> None:
         """Retrieves all logging defined in the providers"""
-        for provider_package, (_, provider) in self._provider_dict.items():
-            if provider.get("logging"):
-                for logging_class_name in provider["logging"]:
-                    if _sanity_check(provider_package, logging_class_name):
+        for provider_package, provider in self._provider_dict.items():
+            if provider.data.get("logging"):
+                for logging_class_name in provider.data["logging"]:
+                    if _sanity_check(provider_package, logging_class_name, provider):
                         self._logging_class_name_set.add(logging_class_name)
 
     def _discover_secrets_backends(self) -> None:
         """Retrieves all secrets backends defined in the providers"""
-        for provider_package, (_, provider) in self._provider_dict.items():
-            if provider.get("secrets-backends"):
-                for secrets_backends_class_name in provider["secrets-backends"]:
-                    if _sanity_check(provider_package, secrets_backends_class_name):
+        for provider_package, provider in self._provider_dict.items():
+            if provider.data.get("secrets-backends"):
+                for secrets_backends_class_name in provider.data["secrets-backends"]:
+                    if _sanity_check(provider_package, secrets_backends_class_name, provider):
                         self._secrets_backend_class_name_set.add(secrets_backends_class_name)
 
     def _discover_auth_backends(self) -> None:
         """Retrieves all API auth backends defined in the providers"""
-        for provider_package, (_, provider) in self._provider_dict.items():
-            if provider.get("auth-backends"):
-                for auth_backend_module_name in provider["auth-backends"]:
-                    if _sanity_check(provider_package, auth_backend_module_name + ".init_app"):
+        for provider_package, provider in self._provider_dict.items():
+            if provider.data.get("auth-backends"):
+                for auth_backend_module_name in provider.data["auth-backends"]:
+                    if _sanity_check(provider_package, auth_backend_module_name + ".init_app", provider):
                         self._api_auth_backend_module_names.add(auth_backend_module_name)
 
     @property
-    def providers(self) -> Dict[str, ProviderInfo]:
+    def providers(self) -> dict[str, ProviderInfo]:
         """Returns information about available providers."""
         self.initialize_providers_list()
         return self._provider_dict
 
     @property
-    def hooks(self) -> Dict[str, Optional[HookInfo]]:
+    def hooks(self) -> MutableMapping[str, HookInfo | None]:
         """
         Returns dictionary of connection_type-to-hook mapping. Note that the dict can contain
         None values if a hook discovered cannot be imported!
         """
         self.initialize_providers_hooks()
         # When we return hooks here it will only be used to retrieve hook information
-        return self._hooks_lazy_dict  # type: ignore
+        return self._hooks_lazy_dict
 
     @property
-    def taskflow_decorators(self) -> Dict[str, Callable]:
+    def taskflow_decorators(self) -> dict[str, TaskDecorator]:
         self.initialize_providers_taskflow_decorator()
         return self._taskflow_decorators
 
     @property
-    def extra_links_class_names(self) -> List[str]:
+    def extra_links_class_names(self) -> list[str]:
         """Returns set of extra link class names."""
         self.initialize_providers_extra_links()
         return sorted(self._extra_link_class_name_set)
 
     @property
-    def connection_form_widgets(self) -> Dict[str, ConnectionFormWidgetInfo]:
-        """Returns widgets for connection forms."""
+    def connection_form_widgets(self) -> dict[str, ConnectionFormWidgetInfo]:
+        """
+        Returns widgets for connection forms.
+        Dictionary keys in the same order that it defined in Hook.
+        """
         self.initialize_providers_hooks()
         self._import_info_from_all_hooks()
         return self._connection_form_widgets
 
     @property
-    def field_behaviours(self) -> Dict[str, Dict]:
+    def field_behaviours(self) -> dict[str, dict]:
         """Returns dictionary with field behaviours for connection types."""
         self.initialize_providers_hooks()
         self._import_info_from_all_hooks()
         return self._field_behaviours
 
     @property
-    def logging_class_names(self) -> List[str]:
+    def logging_class_names(self) -> list[str]:
         """Returns set of log task handlers class names."""
         self.initialize_providers_logging()
         return sorted(self._logging_class_name_set)
 
     @property
-    def secrets_backend_class_names(self) -> List[str]:
+    def secrets_backend_class_names(self) -> list[str]:
         """Returns set of secret backend class names."""
         self.initialize_providers_secrets_backends()
         return sorted(self._secrets_backend_class_name_set)
 
     @property
-    def auth_backend_module_names(self) -> List[str]:
+    def auth_backend_module_names(self) -> list[str]:
         """Returns set of API auth backend class names."""
         self.initialize_providers_auth_backends()
         return sorted(self._api_auth_backend_module_names)
