@@ -14,20 +14,18 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 import datetime
-from typing import Any, Dict, Optional, Union
+from typing import Any, Union
 
-from croniter import CroniterBadCronError, CroniterBadDateError, croniter
 from dateutil.relativedelta import relativedelta
 from pendulum import DateTime
-from pendulum.tz.timezone import Timezone
 
-from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowTimetableInvalid
+from airflow.timetables._cron import CronMixin
 from airflow.timetables.base import DagRunInfo, DataInterval, TimeRestriction, Timetable
-from airflow.utils.dates import cron_presets
-from airflow.utils.timezone import convert_to_utc, make_aware, make_naive
+from airflow.utils.timezone import convert_to_utc
 
 Delta = Union[datetime.timedelta, relativedelta]
 
@@ -40,7 +38,7 @@ class _DataIntervalTimetable(Timetable):
     instance), and schedule a DagRun at the end of each interval.
     """
 
-    def _skip_to_latest(self, earliest: Optional[DateTime]) -> DateTime:
+    def _skip_to_latest(self, earliest: DateTime | None) -> DateTime:
         """Bound the earliest time a run can be scheduled.
 
         This is called when ``catchup=False``. See docstring of subclasses for
@@ -48,8 +46,8 @@ class _DataIntervalTimetable(Timetable):
         """
         raise NotImplementedError()
 
-    def _align(self, current: DateTime) -> DateTime:
-        """Align given time to the scheduled.
+    def _align_to_next(self, current: DateTime) -> DateTime:
+        """Align given time to the next scheduled time.
 
         For fixed schedules (e.g. every midnight); this finds the next time that
         aligns to the declared time, if the given time does not align. If the
@@ -57,6 +55,19 @@ class _DataIntervalTimetable(Timetable):
         """
         raise NotImplementedError()
 
+    def _align_to_prev(self, current: DateTime) -> DateTime:
+        """Align given time to the previous scheduled time.
+
+        For fixed schedules (e.g. every midnight); this finds the prev time that
+        aligns to the declared time, if the given time does not align. If the
+        schedule is not fixed (e.g. every hour), the given time is returned.
+
+        It is not enough to use ``_get_prev(_align_to_next())``, since when a
+        DAG's schedule changes, this alternative would make the first scheduling
+        after the schedule change remain the same.
+        """
+        raise NotImplementedError()
+
     def _get_next(self, current: DateTime) -> DateTime:
         """Get the first schedule after the current time."""
         raise NotImplementedError()
@@ -68,14 +79,14 @@ class _DataIntervalTimetable(Timetable):
     def next_dagrun_info(
         self,
         *,
-        last_automated_data_interval: Optional[DataInterval],
+        last_automated_data_interval: DataInterval | None,
         restriction: TimeRestriction,
-    ) -> Optional[DagRunInfo]:
+    ) -> DagRunInfo | None:
         earliest = restriction.earliest
         if not restriction.catchup:
             earliest = self._skip_to_latest(earliest)
         elif earliest is not None:
-            earliest = self._align(earliest)
+            earliest = self._align_to_next(earliest)
         if last_automated_data_interval is None:
             # First run; schedule the run at the first available time matching
             # the schedule, and retrospectively create a data interval for it.
@@ -83,40 +94,25 @@ class _DataIntervalTimetable(Timetable):
                 return None
             start = earliest
         else:  # There's a previous run.
+            # Alignment is needed when DAG has new schedule interval.
+            align_last_data_interval_end = self._align_to_prev(last_automated_data_interval.end)
             if earliest is not None:
                 # Catchup is False or DAG has new start date in the future.
                 # Make sure we get the later one.
-                start = max(last_automated_data_interval.end, earliest)
+                start = max(align_last_data_interval_end, earliest)
             else:
                 # Data interval starts from the end of the previous interval.
-                start = last_automated_data_interval.end
+                start = align_last_data_interval_end
         if restriction.latest is not None and start > restriction.latest:
             return None
         end = self._get_next(start)
         return DagRunInfo.interval(start=start, end=end)
 
 
-def _is_schedule_fixed(expression: str) -> bool:
-    """Figures out if the schedule has a fixed time (e.g. 3 AM every day).
-
-    :return: True if the schedule has a fixed time, False if not.
-
-    Detection is done by "peeking" the next two cron trigger time; if the
-    two times have the same minute and hour value, the schedule is fixed,
-    and we *don't* need to perform the DST fix.
-
-    This assumes DST happens on whole minute changes (e.g. 12:59 -> 12:00).
-    """
-    cron = croniter(expression)
-    next_a = cron.get_next(datetime.datetime)
-    next_b = cron.get_next(datetime.datetime)
-    return next_b.minute == next_a.minute and next_b.hour == next_a.hour
-
-
-class CronDataIntervalTimetable(_DataIntervalTimetable):
+class CronDataIntervalTimetable(CronMixin, _DataIntervalTimetable):
     """Timetable that schedules data intervals with a cron expression.
 
-    This corresponds to ``schedule_interval=<cron>``, where ``<cron>`` is either
+    This corresponds to ``schedule=<cron>``, where ``<cron>`` is either
     a five/six-segment representation, or one of ``cron_presets``.
 
     The implementation extends on croniter to add timezone awareness. This is
@@ -126,78 +122,18 @@ class CronDataIntervalTimetable(_DataIntervalTimetable):
     Don't pass ``@once`` in here; use ``OnceTimetable`` instead.
     """
 
-    def __init__(self, cron: str, timezone: Timezone) -> None:
-        self._expression = cron_presets.get(cron, cron)
-        self._timezone = timezone
-
     @classmethod
-    def deserialize(cls, data: Dict[str, Any]) -> "Timetable":
+    def deserialize(cls, data: dict[str, Any]) -> Timetable:
         from airflow.serialization.serialized_objects import decode_timezone
 
         return cls(data["expression"], decode_timezone(data["timezone"]))
 
-    def __eq__(self, other: Any) -> bool:
-        """Both expression and timezone should match.
-
-        This is only for testing purposes and should not be relied on otherwise.
-        """
-        if not isinstance(other, CronDataIntervalTimetable):
-            return NotImplemented
-        return self._expression == other._expression and self._timezone == other._timezone
-
-    @property
-    def summary(self) -> str:
-        return self._expression
-
-    def serialize(self) -> Dict[str, Any]:
+    def serialize(self) -> dict[str, Any]:
         from airflow.serialization.serialized_objects import encode_timezone
 
         return {"expression": self._expression, "timezone": encode_timezone(self._timezone)}
 
-    def validate(self) -> None:
-        try:
-            croniter(self._expression)
-        except (CroniterBadCronError, CroniterBadDateError) as e:
-            raise AirflowTimetableInvalid(str(e))
-
-    @cached_property
-    def _should_fix_dst(self) -> bool:
-        # This is lazy so instantiating a schedule does not immediately raise
-        # an exception. Validity is checked with validate() during DAG-bagging.
-        return not _is_schedule_fixed(self._expression)
-
-    def _get_next(self, current: DateTime) -> DateTime:
-        """Get the first schedule after specified time, with DST fixed."""
-        naive = make_naive(current, self._timezone)
-        cron = croniter(self._expression, start_time=naive)
-        scheduled = cron.get_next(datetime.datetime)
-        if not self._should_fix_dst:
-            return convert_to_utc(make_aware(scheduled, self._timezone))
-        delta = scheduled - naive
-        return convert_to_utc(current.in_timezone(self._timezone) + delta)
-
-    def _get_prev(self, current: DateTime) -> DateTime:
-        """Get the first schedule before specified time, with DST fixed."""
-        naive = make_naive(current, self._timezone)
-        cron = croniter(self._expression, start_time=naive)
-        scheduled = cron.get_prev(datetime.datetime)
-        if not self._should_fix_dst:
-            return convert_to_utc(make_aware(scheduled, self._timezone))
-        delta = naive - scheduled
-        return convert_to_utc(current.in_timezone(self._timezone) - delta)
-
-    def _align(self, current: DateTime) -> DateTime:
-        """Get the next scheduled time.
-
-        This is ``current + interval``, unless ``current`` falls right on the
-        interval boundary, when ``current`` is returned.
-        """
-        next_time = self._get_next(current)
-        if self._get_prev(next_time) != current:
-            return next_time
-        return current
-
-    def _skip_to_latest(self, earliest: Optional[DateTime]) -> DateTime:
+    def _skip_to_latest(self, earliest: DateTime | None) -> DateTime:
         """Bound the earliest time a run can be scheduled.
 
         The logic is that we move start_date up until one period before, so the
@@ -218,20 +154,20 @@ class CronDataIntervalTimetable(_DataIntervalTimetable):
             raise AssertionError("next schedule shouldn't be earlier")
         if earliest is None:
             return new_start
-        return max(new_start, self._align(earliest))
+        return max(new_start, self._align_to_next(earliest))
 
     def infer_manual_data_interval(self, *, run_after: DateTime) -> DataInterval:
         # Get the last complete period before run_after, e.g. if a DAG run is
         # scheduled at each midnight, the data interval of a manually triggered
         # run at 1am 25th is between 0am 24th and 0am 25th.
-        end = self._get_prev(self._align(run_after))
+        end = self._align_to_prev(run_after)
         return DataInterval(start=self._get_prev(end), end=end)
 
 
 class DeltaDataIntervalTimetable(_DataIntervalTimetable):
     """Timetable that schedules data intervals with a time delta.
 
-    This corresponds to ``schedule_interval=<delta>``, where ``<delta>`` is
+    This corresponds to ``schedule=<delta>``, where ``<delta>`` is
     either a ``datetime.timedelta`` or ``dateutil.relativedelta.relativedelta``
     instance.
     """
@@ -240,7 +176,7 @@ class DeltaDataIntervalTimetable(_DataIntervalTimetable):
         self._delta = delta
 
     @classmethod
-    def deserialize(cls, data: Dict[str, Any]) -> "Timetable":
+    def deserialize(cls, data: dict[str, Any]) -> Timetable:
         from airflow.serialization.serialized_objects import decode_relativedelta
 
         delta = data["delta"]
@@ -261,9 +197,10 @@ class DeltaDataIntervalTimetable(_DataIntervalTimetable):
     def summary(self) -> str:
         return str(self._delta)
 
-    def serialize(self) -> Dict[str, Any]:
+    def serialize(self) -> dict[str, Any]:
         from airflow.serialization.serialized_objects import encode_relativedelta
 
+        delta: Any
         if isinstance(self._delta, datetime.timedelta):
             delta = self._delta.total_seconds()
         else:
@@ -281,10 +218,13 @@ class DeltaDataIntervalTimetable(_DataIntervalTimetable):
     def _get_prev(self, current: DateTime) -> DateTime:
         return convert_to_utc(current - self._delta)
 
-    def _align(self, current: DateTime) -> DateTime:
+    def _align_to_next(self, current: DateTime) -> DateTime:
         return current
 
-    def _skip_to_latest(self, earliest: Optional[DateTime]) -> DateTime:
+    def _align_to_prev(self, current: DateTime) -> DateTime:
+        return current
+
+    def _skip_to_latest(self, earliest: DateTime | None) -> DateTime:
         """Bound the earliest time a run can be scheduled.
 
         The logic is that we move start_date up until one period before, so the
