@@ -14,17 +14,15 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
-# This product contains a modified portion of 'Flask App Builder' developed by Daniel Vaz Gaspar.
-# (https://github.com/dpgaspar/Flask-AppBuilder).
-# Copyright 2013, Daniel Vaz Gaspar
+from __future__ import annotations
 
 import base64
 import datetime
 import json
 import logging
 import re
-from typing import Any, Dict, List, Optional, Set, Tuple, Type
+from typing import Any
+from uuid import uuid4
 
 from flask import current_app, g, session, url_for
 from flask_appbuilder import AppBuilder
@@ -70,9 +68,13 @@ from flask_jwt_extended import JWTManager, current_user as current_user_jwt
 from flask_login import AnonymousUserMixin, LoginManager, current_user
 from werkzeug.security import check_password_hash, generate_password_hash
 
+from airflow.compat.functools import cached_property
+from airflow.configuration import conf
 from airflow.www.fab_security.sqla.models import Action, Permission, RegisterUser, Resource, Role, User
-from airflow.www.views import ResourceModelView
 
+# This product contains a modified portion of 'Flask App Builder' developed by Daniel Vaz Gaspar.
+# (https://github.com/dpgaspar/Flask-AppBuilder).
+# Copyright 2013, Daniel Vaz Gaspar
 log = logging.getLogger(__name__)
 
 
@@ -82,15 +84,15 @@ def _oauth_tokengetter(token=None):
     from session cookie.
     """
     token = session.get("oauth")
-    log.debug(f"Token Get: {token}")
+    log.debug("Token Get: %s", token)
     return token
 
 
 class AnonymousUser(AnonymousUserMixin):
     """User object used when no active user is logged in."""
 
-    _roles: Set[Tuple[str, str]] = set()
-    _perms: Set[Tuple[str, str]] = set()
+    _roles: set[tuple[str, str]] = set()
+    _perms: set[tuple[str, str]] = set()
 
     @property
     def roles(self):
@@ -132,9 +134,9 @@ class BaseSecurityManager:
     """ Flask-OpenID OpenID """
     oauth = None
     """ Flask-OAuth """
-    oauth_remotes: Dict[str, Any]
+    oauth_remotes: dict[str, Any]
     """ OAuth email whitelists """
-    oauth_whitelists: Dict[str, List] = {}
+    oauth_whitelists: dict[str, list] = {}
     """ Initialized (remote_app) providers dict {'provider_name', OBJ } """
 
     @staticmethod
@@ -146,17 +148,17 @@ class BaseSecurityManager:
 
     oauth_user_info = None
 
-    user_model: Type[User]
+    user_model: type[User]
     """ Override to set your own User Model """
-    role_model: Type[Role]
+    role_model: type[Role]
     """ Override to set your own Role Model """
-    action_model: Type[Action]
+    action_model: type[Action]
     """ Override to set your own Action Model """
-    resource_model: Type[Resource]
+    resource_model: type[Resource]
     """ Override to set your own Resource Model """
-    permission_model: Type[Permission]
+    permission_model: type[Permission]
     """ Override to set your own Permission Model """
-    registeruser_model: Type[RegisterUser]
+    registeruser_model: type[RegisterUser]
     """ Override to set your own RegisterUser Model """
 
     userdbmodelview = UserDBModelView
@@ -199,9 +201,14 @@ class BaseSecurityManager:
     rolemodelview = RoleModelView
     actionmodelview = PermissionModelView
     userstatschartview = UserStatsChartView
-    resourcemodelview = ResourceModelView
     permissionmodelview = PermissionModelView
 
+    @cached_property
+    def resourcemodelview(self):
+        from airflow.www.views import ResourceModelView
+
+        return ResourceModelView
+
     def __init__(self, appbuilder):
         self.appbuilder = appbuilder
         app = self.appbuilder.get_app
@@ -254,7 +261,7 @@ class BaseSecurityManager:
             self.oauth_remotes = {}
             for _provider in self.oauth_providers:
                 provider_name = _provider["name"]
-                log.debug(f"OAuth providers init {provider_name}")
+                log.debug("OAuth providers init %s", provider_name)
                 obj_provider = self.oauth.register(provider_name, **_provider["remote_app"])
                 obj_provider._tokengetter = self.oauth_tokengetter
                 if not self.oauth_user_info:
@@ -298,7 +305,7 @@ class BaseSecurityManager:
         """Returns FAB builtin roles."""
         return self.appbuilder.get_app.config.get("FAB_ROLES", {})
 
-    def get_roles_from_keys(self, role_keys: List[str]) -> Set[RoleModelView]:
+    def get_roles_from_keys(self, role_keys: list[str]) -> set[RoleModelView]:
         """
         Construct a list of FAB role objects, from a list of keys.
 
@@ -318,7 +325,7 @@ class BaseSecurityManager:
                     if fab_role:
                         _roles.add(fab_role)
                     else:
-                        log.warning(f"Can't find role specified in AUTH_ROLES_MAPPING: {fab_role_name}")
+                        log.warning("Can't find role specified in AUTH_ROLES_MAPPING: %s", fab_role_name)
         return _roles
 
     @property
@@ -396,7 +403,7 @@ class BaseSecurityManager:
         return self.appbuilder.get_app.config["AUTH_USER_REGISTRATION_ROLE_JMESPATH"]
 
     @property
-    def auth_roles_mapping(self) -> Dict[str, List[str]]:
+    def auth_roles_mapping(self) -> dict[str, list[str]]:
         """The mapping of auth roles."""
         return self.appbuilder.get_app.config["AUTH_ROLES_MAPPING"]
 
@@ -536,7 +543,7 @@ class BaseSecurityManager:
             ret = f(self, provider, response=response)
             # Checks if decorator is well behaved and returns a dict as supposed.
             if not type(ret) == dict:
-                log.error(f"OAuth user info decorated function did not returned a dict, but: {type(ret)}")
+                log.error("OAuth user info decorated function did not returned a dict, but: %s", type(ret))
                 return {}
             return ret
 
@@ -584,13 +591,13 @@ class BaseSecurityManager:
         if provider == "github" or provider == "githublocal":
             me = self.appbuilder.sm.oauth_remotes[provider].get("user")
             data = me.json()
-            log.debug(f"User info from Github: {data}")
+            log.debug("User info from GitHub: %s", data)
             return {"username": "github_" + data.get("login")}
         # for twitter
         if provider == "twitter":
             me = self.appbuilder.sm.oauth_remotes[provider].get("account/settings.json")
             data = me.json()
-            log.debug(f"User info from Twitter: {data}")
+            log.debug("User info from Twitter: %s", data)
             return {"username": "twitter_" + data.get("screen_name", "")}
         # for linkedin
         if provider == "linkedin":
@@ -598,7 +605,7 @@ class BaseSecurityManager:
                 "people/~:(id,email-address,first-name,last-name)?format=json"
             )
             data = me.json()
-            log.debug(f"User info from Linkedin: {data}")
+            log.debug("User info from LinkedIn: %s", data)
             return {
                 "username": "linkedin_" + data.get("id", ""),
                 "email": data.get("email-address", ""),
@@ -609,7 +616,7 @@ class BaseSecurityManager:
         if provider == "google":
             me = self.appbuilder.sm.oauth_remotes[provider].get("userinfo")
             data = me.json()
-            log.debug(f"User info from Google: {data}")
+            log.debug("User info from Google: %s", data)
             return {
                 "username": "google_" + data.get("id", ""),
                 "first_name": data.get("given_name", ""),
@@ -622,11 +629,11 @@ class BaseSecurityManager:
         # https://docs.microsoft.com/en-us/azure/active-directory/develop/
         # active-directory-protocols-oauth-code
         if provider == "azure":
-            log.debug(f"Azure response received : {resp}")
+            log.debug("Azure response received : %s", resp)
             id_token = resp["id_token"]
             log.debug(str(id_token))
             me = self._azure_jwt_token_parse(id_token)
-            log.debug(f"Parse JWT token : {me}")
+            log.debug("Parse JWT token : %s", me)
             return {
                 "name": me.get("name", ""),
                 "email": me["upn"],
@@ -640,7 +647,7 @@ class BaseSecurityManager:
         if provider == "openshift":
             me = self.appbuilder.sm.oauth_remotes[provider].get("apis/user.openshift.io/v1/users/~")
             data = me.json()
-            log.debug(f"User info from OpenShift: {data}")
+            log.debug("User info from OpenShift: %s", data)
             return {"username": "openshift_" + data.get("metadata").get("name")}
         # for Okta
         if provider == "okta":
@@ -813,7 +820,7 @@ class BaseSecurityManager:
         if self.auth_role_admin not in self.builtin_roles:
             self.add_role(self.auth_role_admin)
         self.add_role(self.auth_role_public)
-        if self.count_users() == 0:
+        if self.count_users() == 0 and self.auth_role_public != self.auth_role_admin:
             log.warning(LOGMSG_WAR_SEC_NO_USER)
 
     def reset_password(self, userid, password):
@@ -854,6 +861,14 @@ class BaseSecurityManager:
             user.fail_login_count += 1
         self.update_user(user)
 
+    def _rotate_session_id(self):
+        """
+        Upon successful authentication when using the database session backend,
+        we need to rotate the session id
+        """
+        if conf.get('webserver', 'SESSION_BACKEND') == 'database':
+            session.sid = str(uuid4())
+
     def auth_user_db(self, username, password):
         """
         Method for authenticating user, auth db style
@@ -878,6 +893,7 @@ class BaseSecurityManager:
             log.info(LOGMSG_WAR_SEC_LOGIN_FAILED.format(username))
             return None
         elif check_password_hash(user.password, password):
+            self._rotate_session_id()
             self.update_user_auth_stat(user, True)
             return user
         else:
@@ -914,12 +930,12 @@ class BaseSecurityManager:
 
         # perform the LDAP search
         log.debug(
-            f"LDAP search for '{filter_str}' with fields {request_fields} in scope '{self.auth_ldap_search}'"
+            "LDAP search for %r with fields %s in scope %r", filter_str, request_fields, self.auth_ldap_search
         )
         raw_search_result = con.search_s(
             self.auth_ldap_search, ldap.SCOPE_SUBTREE, filter_str, request_fields
         )
-        log.debug(f"LDAP search returned: {raw_search_result}")
+        log.debug("LDAP search returned: %s", raw_search_result)
 
         # Remove any search referrals from results
         search_result = [
@@ -929,8 +945,9 @@ class BaseSecurityManager:
         # only continue if 0 or 1 results were returned
         if len(search_result) > 1:
             log.error(
-                f"LDAP search for '{filter_str}' in scope "
-                f"'{self.auth_ldap_search!a}' returned multiple results"
+                "LDAP search for %r in scope '%a' returned multiple results",
+                self.auth_ldap_search,
+                filter_str,
             )
             return None, None
 
@@ -944,7 +961,7 @@ class BaseSecurityManager:
         except (IndexError, NameError):
             return None, None
 
-    def _ldap_calculate_user_roles(self, user_attributes: Dict[str, List[bytes]]) -> List[str]:
+    def _ldap_calculate_user_roles(self, user_attributes: dict[str, list[bytes]]) -> list[str]:
         user_role_objects = set()
 
         # apply AUTH_ROLES_MAPPING
@@ -961,7 +978,7 @@ class BaseSecurityManager:
             if fab_role:
                 user_role_objects.add(fab_role)
             else:
-                log.warning(f"Can't find AUTH_USER_REGISTRATION role: {registration_role_name}")
+                log.warning("Can't find AUTH_USER_REGISTRATION role: %s", registration_role_name)
 
         return list(user_role_objects)
 
@@ -976,9 +993,9 @@ class BaseSecurityManager:
         assert self.auth_ldap_bind_user, "AUTH_LDAP_BIND_USER must be set"
 
         try:
-            log.debug(f"LDAP bind indirect TRY with username: '{self.auth_ldap_bind_user}'")
+            log.debug("LDAP bind indirect TRY with username: %r", self.auth_ldap_bind_user)
             con.simple_bind_s(self.auth_ldap_bind_user, self.auth_ldap_bind_password)
-            log.debug(f"LDAP bind indirect SUCCESS with username: '{self.auth_ldap_bind_user}'")
+            log.debug("LDAP bind indirect SUCCESS with username: %r", self.auth_ldap_bind_user)
         except ldap.INVALID_CREDENTIALS as ex:
             log.error("AUTH_LDAP_BIND_USER and AUTH_LDAP_BIND_PASSWORD are not valid LDAP bind credentials")
             raise ex
@@ -987,21 +1004,21 @@ class BaseSecurityManager:
     def _ldap_bind(ldap, con, dn: str, password: str) -> bool:
         """Validates/binds the provided dn/password with the LDAP sever."""
         try:
-            log.debug(f"LDAP bind TRY with username: '{dn}'")
+            log.debug("LDAP bind TRY with username: %r", dn)
             con.simple_bind_s(dn, password)
-            log.debug(f"LDAP bind SUCCESS with username: '{dn}'")
+            log.debug("LDAP bind SUCCESS with username: %r", dn)
             return True
         except ldap.INVALID_CREDENTIALS:
             return False
 
     @staticmethod
-    def ldap_extract(ldap_dict: Dict[str, List[bytes]], field_name: str, fallback: str) -> str:
+    def ldap_extract(ldap_dict: dict[str, list[bytes]], field_name: str, fallback: str) -> str:
         raw_value = ldap_dict.get(field_name, [bytes()])
         # decode - if empty string, default to fallback, otherwise take first element
         return raw_value[0].decode("utf-8") or fallback
 
     @staticmethod
-    def ldap_extract_list(ldap_dict: Dict[str, List[bytes]], field_name: str) -> List[str]:
+    def ldap_extract_list(ldap_dict: dict[str, list[bytes]], field_name: str) -> list[str]:
         raw_list = ldap_dict.get(field_name, [])
         # decode - removing empty strings
         return [x.decode("utf-8") for x in raw_list if x.decode("utf-8")]
@@ -1149,7 +1166,7 @@ class BaseSecurityManager:
             # Sync the user's roles
             if user and user_attributes and self.auth_roles_sync_at_login:
                 user.roles = self._ldap_calculate_user_roles(user_attributes)
-                log.debug(f"Calculated new roles for user='{user_dn}' as: {user.roles}")
+                log.debug("Calculated new roles for user=%r as: %s", user_dn, user.roles)
 
             # If the user is new, register them
             if (not user) and user_attributes and self.auth_user_registration:
@@ -1164,7 +1181,7 @@ class BaseSecurityManager:
                     ),
                     role=self._ldap_calculate_user_roles(user_attributes),
                 )
-                log.debug(f"New user registered: {user}")
+                log.debug("New user registered: %s", user)
 
                 # If user registration failed, go away
                 if not user:
@@ -1173,6 +1190,7 @@ class BaseSecurityManager:
 
             # LOGIN SUCCESS (only if user is now registered)
             if user:
+                self._rotate_session_id()
                 self.update_user_auth_stat(user)
                 return user
             else:
@@ -1200,6 +1218,7 @@ class BaseSecurityManager:
             log.info(LOGMSG_WAR_SEC_LOGIN_FAILED.format(email))
             return None
         else:
+            self._rotate_session_id()
             self.update_user_auth_stat(user)
             return user
 
@@ -1229,10 +1248,11 @@ class BaseSecurityManager:
             log.info(LOGMSG_WAR_SEC_LOGIN_FAILED.format(username))
             return None
 
+        self._rotate_session_id()
         self.update_user_auth_stat(user)
         return user
 
-    def _oauth_calculate_user_roles(self, userinfo) -> List[str]:
+    def _oauth_calculate_user_roles(self, userinfo) -> list[str]:
         user_role_objects = set()
 
         # apply AUTH_ROLES_MAPPING
@@ -1256,7 +1276,7 @@ class BaseSecurityManager:
             if fab_role:
                 user_role_objects.add(fab_role)
             else:
-                log.warning(f"Can't find AUTH_USER_REGISTRATION role: {registration_role_name}")
+                log.warning("Can't find AUTH_USER_REGISTRATION role: %s", registration_role_name)
 
         return list(user_role_objects)
 
@@ -1273,7 +1293,7 @@ class BaseSecurityManager:
         elif "email" in userinfo:
             username = userinfo["email"]
         else:
-            log.error(f"OAUTH userinfo does not have username or email {userinfo}")
+            log.error("OAUTH userinfo does not have username or email %s", userinfo)
             return None
 
         # If username is empty, go away
@@ -1294,7 +1314,7 @@ class BaseSecurityManager:
         # Sync the user's roles
         if user and self.auth_roles_sync_at_login:
             user.roles = self._oauth_calculate_user_roles(userinfo)
-            log.debug(f"Calculated new roles for user='{username}' as: {user.roles}")
+            log.debug("Calculated new roles for user=%r as: %s", username, user.roles)
 
         # If the user is new, register them
         if (not user) and self.auth_user_registration:
@@ -1305,15 +1325,16 @@ class BaseSecurityManager:
                 email=userinfo.get("email", "") or f"{username}@email.notfound",
                 role=self._oauth_calculate_user_roles(userinfo),
             )
-            log.debug(f"New user registered: {user}")
+            log.debug("New user registered: %s", user)
 
             # If user registration failed, go away
             if not user:
-                log.error(f"Error creating a new OAuth user {username}")
+                log.error("Error creating a new OAuth user %s", username)
                 return None
 
         # LOGIN SUCCESS (only if user is now registered)
         if user:
+            self._rotate_session_id()
             self.update_user_auth_stat(user)
             return user
         else:
@@ -1328,8 +1349,8 @@ class BaseSecurityManager:
         return False
 
     def _get_user_permission_resources(
-        self, user: Optional[User], action_name: str, resource_names: Optional[List[str]] = None
-    ) -> Set[str]:
+        self, user: User | None, action_name: str, resource_names: list[str] | None = None
+    ) -> set[str]:
         """
         Return a set of resource names with a certain action name
         that a user has access to. Mainly used to fetch all menu permissions
@@ -1361,7 +1382,7 @@ class BaseSecurityManager:
         result.update(role_resource_names)
         return result
 
-    def get_user_menu_access(self, menu_names: Optional[List[str]] = None) -> Set[str]:
+    def get_user_menu_access(self, menu_names: list[str] | None = None) -> set[str]:
         if current_user.is_authenticated:
             return self._get_user_permission_resources(g.user, "menu_access", resource_names=menu_names)
         elif current_user_jwt:
@@ -1484,7 +1505,7 @@ class BaseSecurityManager:
         """Generic function that returns all existing users"""
         raise NotImplementedError
 
-    def get_role_permissions_from_db(self, role_id: int) -> List[Permission]:
+    def get_role_permissions_from_db(self, role_id: int) -> list[Permission]:
         """Get all DB permissions from a role id"""
         raise NotImplementedError
 
@@ -1530,11 +1551,11 @@ class BaseSecurityManager:
         """
         raise NotImplementedError
 
-    def filter_roles_by_perm_with_action(self, permission_name: str, role_ids: List[int]):
+    def filter_roles_by_perm_with_action(self, permission_name: str, role_ids: list[int]):
         raise NotImplementedError
 
     def permission_exists_in_one_or_more_roles(
-        self, resource_name: str, action_name: str, role_ids: List[int]
+        self, resource_name: str, action_name: str, role_ids: list[int]
     ) -> bool:
         """Finds and returns permission views for a group of roles"""
         raise NotImplementedError
