@@ -14,6 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 THIRD_PARTY_INDEXES = {
     'boto3': 'https://boto3.amazonaws.com/v1/documentation/api/latest',
@@ -24,7 +25,7 @@ THIRD_PARTY_INDEXES = {
     'mongodb': 'https://pymongo.readthedocs.io/en/3.11.3',
     'pandas': 'https://pandas.pydata.org/pandas-docs/stable',
     'python': 'https://docs.python.org/3',
-    'requests': 'https://requests.readthedocs.io/en/master',
+    'requests': 'https://requests.readthedocs.io/en/stable',
     'sqlalchemy': 'https://docs.sqlalchemy.org/en/latest',
     'google-api-core': 'https://googleapis.dev/python/google-api-core/latest',
     'google-cloud-automl': 'https://googleapis.dev/python/automl/latest',
